#!/usr/bin/python3
# VERSION 0.0.3

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import numpy as np
import os
import yaml

POINTS_PER_INSTANCE = 1.0
THRESHOLD = 0.001


def load_output(file):
    raw_dict = yaml.load("{" + file.readline() + "}")
    for name in raw_dict:
        raw_dict[name] = np.array(raw_dict[name], dtype=np.float64)
    return raw_dict


def compare_outputs(correct: dict, tested: dict):
    for k in correct:
        if k not in tested:
            raise ValueError("Missing variable: {}".format(k))

        errors = np.abs(correct[k] - tested[k])
        correct[k][correct[k] == 0] = 1.0
        realtive_mean_error = (errors / correct[k]).mean()
        if realtive_mean_error > THRESHOLD:
            return False
    return True


if __name__ == "__main__":
    parser = ArgumentParser(description="Optil.io judge for Automatic Differentiation.",
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument(dest="instance_input", metavar="instance-input", help="instance input filepath")
    parser.add_argument(dest="user_output", metavar="user-output", help="user output filepath")
    parser.add_argument(dest="instance_output", metavar="instance-output", help="correcte instance output filepath")
    parser.add_argument(dest="usertime", help="IGNORE")
    parser.add_argument(dest="perftime", help="IGNORE")

    args = parser.parse_args()

    results = []
    if not os.path.exists(args.user_output):
        print("0|No user output file.")
        exit(0)
    else:

        with open(args.instance_input) as input_file, \
                open(args.instance_output) as output_file, \
                open(args.user_output) as user_file:

            num_instances = int(input_file.readline())
            for _ in range(num_instances):
                # Ignore vars/params/function lines:
                for _ in range(3):
                    input_file.readline()
                # Read number of tests for this instance:
                num_tests = int(input_file.readline())
                try:
                    for _ in range(num_tests):
                        # ignore input file line:
                        input_file.readline()
                        user_output = load_output(user_file)
                        correct_output = load_output(output_file)

                        results.append(compare_outputs(user_output, correct_output))
                except AssertionError:
                    print("0|Error while parsing user output.")
                    exit(0)

    score = float((np.array(results)).sum() / len(results) * POINTS_PER_INSTANCE)
    result_string = "SUCCESS"
    print("{:0.2f}|SUCCESS".format(score, result_string))
